/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
#pragma once

#include "array_with_size.hpp"
#include "tslog.hpp"

#include <boost/utility/string_ref.hpp>
#include <cstdlib>

/* kdbus driver hard limit for names per connection - as in kernel's kdbus/limits.h. */
#define KDBUS_CONN_MAX_NAMES 256

class KdbusBusNames : public ArrayWithSize<boost::string_ref, KDBUS_CONN_MAX_NAMES>
{
public:
	KdbusBusNames &addSpaceSeparatedNames(const char *names) noexcept {
		if (names) {
			boost::string_ref current(names);
			while (!current.empty()) {

				if (!(isalnum(current[0]) || current[0] == ' '))
					tslog::log_verbose("Wrong name(", current, "): ", names, "\n");

				auto space = current.find_first_of(" ");
				push_back(current.substr(0, space));

				if (space != current.npos)	// skip the space
					current = current.substr(space+1);
				else
					current.clear();
			}
		}
		return *this;
	}
};
