/*
 * Copyright (c) 2018-2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
#include "groups_proxy.hpp"
#include "tslog.hpp"
#include <array>
#include <grp.h>
#include <pwd.h>

/**
 * \file
 * \ingroup Implementation
 */

std::vector<gid_t> get_groups(const uid_t uid, const gid_t gid) {
	std::array <char, 1024> buf;
	struct passwd *user_pw = nullptr;
	struct passwd pwd;

	if (getpwuid_r(uid, &pwd, buf.data(), buf.size(), &user_pw)) {
		tslog::log_error("getpwuid_r failed uid:", uid, " gid: ", gid, "\n");
		return {};
	}
	if (!user_pw) {
		tslog::log_error("getpwuid_r failed. no matching password record was found uid:", uid, " gid: ", gid, "\n");
		return {};
	}

	int ngroups = 0;
	// just get number of groups
	getgrouplist(user_pw->pw_name, gid, NULL, &ngroups);
	// getgrouplist returns -1 and ngroups contains number of groups

	if (ngroups == 0)
		return {};

	auto groups = std::vector<gid_t> (ngroups);
	int r = getgrouplist(user_pw->pw_name, gid, groups.data(), &ngroups);
	if (r < 0) {
		tslog::log_error("getgrouplist failed uid:", uid, " gid: ", gid, "\n");
		groups.clear();
	}
	return groups;
}
