/*
 * Copyright (c) 2018-2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/


/**
 * \file
 * \ingroup Implementation
 */

#ifndef _OWN_TREE_H
#define _OWN_TREE_H

#include "policy.hpp"
#include <memory>
#include <unordered_map>
#include <deque>

namespace ldp_xml_parser
{

	class TreeNode{
	public:
		explicit TreeNode(const std::string &token);
		void add(std::deque<std::string>& tokens, const DecisionItem& decision, const bool is_prefix);
		void setDecisionItem(const DecisionItem& decision, const bool is_prefix);
		const std::string &getToken() const;
		const DecisionItem &getOwnPrefixDecisionItem() const;
		const DecisionItem &getOwnDecisionItem() const;
		const std::unordered_map<std::string, std::shared_ptr<TreeNode>> &getChildren() const;
	private:
		DecisionItem prefixBasedDecision() const;

		std::string __token;	// element of dot-separated name
		DecisionItem __own_prefix_decision_item;
		DecisionItem __own_decision_item;
		std::unordered_map<std::string, std::shared_ptr<TreeNode>> __children;
		friend class OwnershipTree;
	};


	class OwnershipTree{
	public:
		OwnershipTree();
		void addItem(const ItemOwn &item);
		const std::shared_ptr<TreeNode> getRoot() const { return __root; };
	private:
		std::shared_ptr<TreeNode> __root;
	};
}
#endif
