#include "internal/naive_policy_checker.hpp"
#include "internal/tslog.hpp"

using ldp_xml_parser::MatchItemOwn;
using ldp_xml_parser::DecisionResult;

struct OwnershipTest {
	DecisionResult expected_result;
	uid_t user;
	gid_t group;
	const char* label;
	const char* service;
};

const int ROOT = 0;
const int GUEST = 9999;
const int GUEST1 = 9991;
const int GUEST2 = 9992;
const int GUEST12 = 9993;

/**
 * This test set tests ability to parse xml db
 * and check ownership privilege in many use cases
 * including prefix feature
 */
struct OwnershipTest ownership_tests[]={
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.ldpo.a"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.ldpoga"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.ldpogd"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.ldposa"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.ldpo.any_suffix"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.ldpnotexistent"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.ldponotexistent"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.z"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.zz"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.aa"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.aaa"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a1"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a1.b"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a1.b1"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a1.c"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.b"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.b.c"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.b.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.c"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.c.c"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.c.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.zz"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.bsth"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.bsthelse"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c.z.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c.d"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c.d.z"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c.d.e"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c.d.esth"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c.d.ee"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c.d.e.z"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c.d.e.z.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c.d.e.f"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1.z.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1.d"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1.d.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1.d.e"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1.d.esth"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1.d.ee"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1.d.e.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1.d.e.z.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1.d.e.f"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c1.d.e.f.g"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c2"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c2.d"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c2.d.z"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c2.dd"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c2.d.e"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c2.d.e.f"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c2.d.e.f.z"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c2.d.e.fsth"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3.z.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3.d"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3.d.z"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3.d.e"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3.d.esth"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3.d.ee"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3.d.e.z"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3.d.e.z.z"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3.d.e.f"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.b.c3.d.e.f.g"},
	{DecisionResult::DENY, ROOT, ROOT, "User::Shell", "org.tizen.a.c"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.c.z"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.d"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.a.d.z"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.pok"},
	{DecisionResult::ALLOW, ROOT, ROOT, "User::Shell", "org.tizen.pnope"},
	{DecisionResult::ALLOW, GUEST, GUEST, "User::Shell", "org.tizen.pok"},
	{DecisionResult::DENY, GUEST, GUEST, "User::Shell", "org.tizen.pnope"},
	{DecisionResult::ALLOW, GUEST, GUEST, "User::Shell", "org.tizen.pok1"},
	{DecisionResult::ALLOW, GUEST, GUEST, "User::Shell", "org.tizen.pok1.z"},
	{DecisionResult::DENY, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a"},
	{DecisionResult::ALLOW, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a.b1"},
	{DecisionResult::ALLOW, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a.b1.z"},
	{DecisionResult::ALLOW, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a.b2"},
	{DecisionResult::DENY, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a.b2.z"},
	{DecisionResult::DENY, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a.b3"},
	{DecisionResult::DENY, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a.b3.z"},
	{DecisionResult::DENY, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a.b4"},
	{DecisionResult::DENY, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a.b4.z"},
	{DecisionResult::DENY, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a1"},
	{DecisionResult::ALLOW, GUEST, GUEST, "User::Shell", "org.tizen.pok1.a1.z"},
	{DecisionResult::ALLOW, GUEST1, GUEST1, "User::Shell", "org.tizen.pok2"},
	{DecisionResult::DENY, GUEST2, GUEST2, "User::Shell", "org.tizen.pok2"},
	{DecisionResult::ALLOW, GUEST12, GUEST12, "User::Shell", "org.tizen.pok2"},
	{DecisionResult::DENY, GUEST1, GUEST1, "User::Shell", "org.tizen.pok2.a"},
	{DecisionResult::DENY, GUEST2, GUEST2, "User::Shell", "org.tizen.pok2.a"},
	{DecisionResult::DENY, GUEST12, GUEST12, "User::Shell", "org.tizen.pok2.a"},
	{DecisionResult::DENY, GUEST1, GUEST1, "User::Shell", "org.tizen.pok2.a.b"},
	{DecisionResult::ALLOW, GUEST2, GUEST2, "User::Shell", "org.tizen.pok2.a.b"},
	{DecisionResult::ALLOW, GUEST12, GUEST12, "User::Shell", "org.tizen.pok2.a.b"}
};

std::map<DecisionResult, const char*> DECISIONS {
		{DecisionResult::CYNARA_ERROR, "CYNARA_ERROR"},
		{DecisionResult::DENY, "DENY"},
		{DecisionResult::ALLOW, "ALLOW"}};

void ownershipTest_print(struct OwnershipTest* t, DecisionResult result) {
	printf("uid = %lu, gid = %lu, label = %s, service = %s, expected = %s, result = %s",
		   (unsigned long)t->user, (unsigned long)t->group, t->label, t->service,
		   DECISIONS[t->expected_result], DECISIONS[result]);
}

bool ownership_test() {
	unsigned  i = 0;
	bool flag = true;
	policy_checker_system().initDb("tests/default_deny/system.conf");
	for (i = 0; i < sizeof(ownership_tests)/sizeof(struct OwnershipTest); i++) {
		auto decision = policy_checker_system().check(ownership_tests[i].user,
													  ownership_tests[i].group,
													  ownership_tests[i].label,
													  MatchItemOwn(ownership_tests[i].service));
		if (ownership_tests[i].expected_result != decision) {
			printf("[ERROR][%d] ownership test failed: %s %s ",
					i, DECISIONS[ownership_tests[i].expected_result], DECISIONS[decision]);
			ownershipTest_print(&ownership_tests[i], decision);
			printf("\n");
			flag = false;
		}
	}
	return flag;
}

int main() {
	tslog::init();
	if (!ownership_test())
		return -1;
return 0;
}
