#include "internal/naive_policy_checker.hpp"
#include "internal/tslog.hpp"

using ldp_xml_parser::MatchItemOwn;
using ldp_xml_parser::DecisionResult;

struct OwnershipTest {
	DecisionResult expected_result;
	uid_t user;
	gid_t group;
	const char* label;
	const char* service;
};
/**
 * This test set tests ability to parse xml db
 * and check ownership privilege in many use cases
 * including prefix feature
 */
struct OwnershipTest ownership_tests[]={
	{DecisionResult::ALLOW,     0,    0, "test", "org.test.test1"},
	{DecisionResult::ALLOW,  5009,    0, "test", "org.test.test1"},

	{DecisionResult::DENY,      0,    0, "test", "org.test.test2"},
	{DecisionResult::DENY,   5009,    0, "test", "org.test.test2"},

	{DecisionResult::DENY,      0,    0, "test", "org.test.test3"},
	{DecisionResult::DENY,   5009,    0, "test", "org.test.test3"},

	{DecisionResult::DENY,      0,    0, "test", "org.test.test4"},
	{DecisionResult::ALLOW,  5009,    0, "test", "org.test.test4"},

	{DecisionResult::DENY,      0,    0, "test", "org.test.test5"},

	{DecisionResult::ALLOW,     0,    0, "test", "org.test.test6"},

	{DecisionResult::ALLOW,     0,    0, "test", "org.test.test7"},

	{DecisionResult::ALLOW,     0,    0, "test", "a.b.c"},
	{DecisionResult::ALLOW,     0,    0, "test", "a.b"},
	{DecisionResult::DENY,      0,    0, "test", "c"},
	{DecisionResult::DENY,      0,    0, "test", "a.c"},
	{DecisionResult::DENY,      0,    0, "test", "b.c"},

	{DecisionResult::ALLOW, 1000,     0, "test", "org.test.testX"},
	{DecisionResult::ALLOW, 1000,     0, "test", "org.test.testY"}
};

std::map<DecisionResult, const char*> DECISIONS {
		{DecisionResult::CYNARA_ERROR, "CYNARA_ERROR"},
		{DecisionResult::DENY, "DENY"},
		{DecisionResult::ALLOW, "ALLOW"}};

void ownershipTest_print(struct OwnershipTest* t, DecisionResult result) {
	printf("uid = %lu, gid = %lu, label = %s, service = %s, expected = %s, result = %s",
		   (unsigned long)t->user, (unsigned long)t->group, t->label,
		   t->service, DECISIONS[t->expected_result], DECISIONS[result]);
}

bool ownership_test() {
	unsigned  i = 0;
	bool flag = true;
	policy_checker_system().initDb("tests/default_allow/system.conf");
	for (i = 0; i < sizeof(ownership_tests)/sizeof(struct OwnershipTest); i++) {
		auto decision = policy_checker_system().check(ownership_tests[i].user,
													  ownership_tests[i].group,
													  ownership_tests[i].label,
													  MatchItemOwn(ownership_tests[i].service));
		if (ownership_tests[i].expected_result != decision) {
			printf("[ERROR][%d] ownership test failed: %s %s ",
					i, DECISIONS[ownership_tests[i].expected_result], DECISIONS[decision]);
			ownershipTest_print(&ownership_tests[i], decision);
			printf("\n");
			flag = false;
		}
	}
	return flag;
}

int main() {
	tslog::init();
	if (!ownership_test())
		return -1;
return 0;
}
