/* This file is part of test-runner (see template.c)
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 * Author: Kazimierz Krosman <k.krosman@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

#ifndef TEST_RUNNER_H
#define TEST_RUNNER_H
#include <stdbool.h>

#define TC_NAME "libdbuspolicy-tests"
#define TEST_PATH "/usr/lib/dbus-tests/test-suites/libdbuspolicy-tests/"
#define CYNARA_PATH TEST_PATH "cynara_prepare.sh"
#define MAX_TC_NUM 1024
#define MAX_BUFFER (64*1024)
#define MAX_COMMENT 1024

enum {
	INIT_TEST,
	NEW_STDOUT,
	NEW_STDERR,
	RESULT_CODE,
	RESULT_SIGNAL,
	RESULT_ERROR,
	RESULT_TIMEOUT
};

struct test_result {
	bool is_positive;
	char comment[MAX_COMMENT];
	char result[MAX_COMMENT];
	char name[MAX_COMMENT];
};

struct test_case {
	const char* name;
	const char* description;
};

struct binary {
	const char* path;
	const char* name;
	/* can be filled by asking binary */
	struct test_case* test_cases;
	int timeout;

	char** (*prepare_args)(const struct binary* b, const char* test_name);
	void (*parse)(const struct binary* b, const char* test_name, char* buffer, int state_change, int state_option);
	int (*init)(void);
	int (*clean)(void);
};

char* get_test_id(char *dest, size_t dest_len, const struct binary* b, const char* test_name);
void add_test_result(const char* test_id, const char* result, const char* comment, int res);


#endif /* TEST_RUNNER_H */
